#!/usr/bin/env ts-node
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// tslint:disable:no-shadowed-variable
import test from 'blue-tape';
import sinon from 'sinon';
import ThrottleQueue from './throttle-queue';
const EXPECTED_ITEM1 = { test: 'testing123' };
const EXPECTED_ITEM2 = { mol: 42 };
const EXPECTED_ITEM3 = 42;
const THROTTLE_PERIOD_TIME = 10; // milliseconds
test('ThrottleQueue 1 item', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const q = new ThrottleQueue(THROTTLE_PERIOD_TIME);
    const spy = sinon.spy();
    q.subscribe(spy);
    q.next(EXPECTED_ITEM1);
    t.ok(spy.calledOnce, 'should called right after first item');
    t.deepEqual(spy.firstCall.args[0], EXPECTED_ITEM1, 'should get the first item immediately');
}));
test('ThrottleQueue 2 item', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const q = new ThrottleQueue(THROTTLE_PERIOD_TIME);
    const spy = sinon.spy();
    q.subscribe(spy);
    q.next(EXPECTED_ITEM1);
    q.next(EXPECTED_ITEM2);
    t.ok(spy.calledOnce, 'should only be called once right after next two items');
    t.deepEqual(spy.firstCall.args[0], EXPECTED_ITEM1, 'should get the first item');
    yield new Promise(resolve => setTimeout(resolve, THROTTLE_PERIOD_TIME + 3));
    t.ok(spy.calledOnce, 'should drop the second call after period because of throttle');
}));
test('ThrottleQueue 3 items', (t) => __awaiter(void 0, void 0, void 0, function* () {
    const q = new ThrottleQueue(THROTTLE_PERIOD_TIME);
    const spy = sinon.spy();
    q.subscribe(spy);
    q.next(EXPECTED_ITEM1);
    q.next(EXPECTED_ITEM2);
    yield new Promise(resolve => setTimeout(resolve, THROTTLE_PERIOD_TIME + 3));
    q.next(EXPECTED_ITEM3);
    t.ok(spy.calledTwice, 'should received the third item after THROTTLE_TIME');
    t.deepEqual(spy.secondCall.args[0], EXPECTED_ITEM3, 'should received EXPECTED_ITEM3 (not the ITEM2!)');
}));
//# sourceMappingURL=throttle-queue.spec.js.map